<?php
/**
 * Copyright (c) 2010, Gareth Bond, http://www.gazbond.co.uk
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *     following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials provided with the distribution.
 *   * Neither the name of Yii Software LLC nor the names of its contributors may be used to endorse or
 *     promote products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
class CSSVars extends CApplicationComponent
{	
	public $privatePath='application.css';
	public $publicPath='webroot.css';
	private $values;

	public function init()
	{
		$phpPath=Yii::getPathOfAlias($this->privatePath);
		$cssPath=Yii::getPathOfAlias($this->publicPath);
		$files=CFileHelper::findFiles($phpPath,array('fileTypes'=>array('css')));
		foreach($files as $file)
		{
			$content=$this->parse($file);
			$savePathFull=$cssPath.str_replace($phpPath,'',$file);
			$savePath=dirname($savePathFull);			

			if(!file_exists($savePath))
			{
				mkdir($savePath,0777,true);
			}

			$handle=fopen($savePathFull,'w');
			fwrite($handle,$content);
			fclose($handle);
		}
		parent::init();
	}

	private function parse($file)
	{
		$lines=file($file);
		$content='';
		foreach($lines as $line)
		{
			$content.=$this->findAndReplaceVars($line);			
		}
		return $content;
	}

	private function findAndReplaceVars($line)
	{
		preg_match_all('/\s*\\$([A-Za-z1-9_\-]+)(\s*:\s*(.*?);)?\s*/',$line,$vars);
		$found=$vars[0];
		$varNames=$vars[1];
		$varValues=$vars[3];
		$count=count($found);
		for($i=0; $i<$count; $i++)
		{
			$varName=trim($varNames[$i]);
			$varValue=trim($varValues[$i]);
			if ($varValue)
			{
				$this->values[$varName]=$this->findAndReplaceVars($varValue);
			}
			else if(isset($this->values[$varName]))
			{
				$line=preg_replace('/\\$'.$varName.'(\W|\z)/',$this->values[$varName].'\\1',$line);
			}
		}
		$line=str_replace($found,'',$line);
		return $line;
	}
}
